
# q-expansion of newform 882.6.a.s, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 882
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[4], [0], [26], [0], [-664], [-318], [1582], [-236], [-2212], [4954], [7128], [4358], [10542], [-8452], [5352], [33354], [-15436], [36762], [40972], [9092], [73454], [89400], [-6428], [-122658], [-21370], [-36814], [-104528], [-214440], [28798], [56014], [185400], [64532], [-152930], [343460], [174858], [-452552], [499066], [-475588], [120224], [508874], [-487560], [544410], [-376404], [844946], [492794], [914776], [311780], [1287760], [1289052], [-678214], [1117310], [1261956], [-948218], [-486396], [-1039098], [-1351044], [-1118110], [190104], [-200506], [-1092370], [-1812580], [2100306], [1641044], [-945232], [-415354], [-1184814], [1371548], [963522], [-2577312], [3067514], [-3101442], [327508], [2867392], [3580294], [1642348], [-2056976], [-616142], [-2192118], [-3284538], [3612190], [5414892], [3606294], [2782140], [-6276194], [-641592], [-6055456], [5166814], [-227798], [585146], [-3414536], [716300], [5240920], [1117016], [-1344576], [-6546476], [-8220504], [-5110446], [9699990], [3172948], [-6625746], [3847068], [-5001758], [2277724], [-8869794], [14010180], [-8753266], [-10611692], [1885518], [-12725628], [-7188458], [-10849440], [-4905114], [-2584450], [4993364], [-11821936], [5470070], [-9649340], [292368], [-6940806], [13291232], [-2052190], [-15703918], [-969534], [14990784], [7160380], [91834], [22098134], [15838800], [-6314176], [-6930030], [14233116], [5944596], [-682752], [14633286], [-11636738], [-19147186], [-5392606], [-3043476], [22965210], [-19078682], [-10941436], [-21259366], [-14225648], [-2761032], [38214674], [10604416], [40700866], [-31012002], [-10910372], [-10408940], [16406350], [14857662], [-27201772], [27124176], [-8422692], [-30863748], [4938952], [5625750], [-26007338], [3021602], [34828192], [9390086], [14476832], [9249756], [49778022], [-8956008], [26240048], [-28050638], [-39818590], [30907458], [-30760424], [40573178], [4413684], [-65173014], [-19766640], [-1504714], [-22669932], [-37958062], [-44057648], [-20219702], [17062584], [-9671372], [60771670], [-44741482], [66552244], [47710578], [-6424626], [5064124], [15586218], [69132864], [-79152586], [-3748344], [-27450476], [-21603830], [81910272], [-10668794], [44039234], [28403182], [16705110], [36458352], [46633986], [73467456], [40596010], [47802150], [-48230148], [-6978822], [90110224], [-58214160], [81919590], [-19894388], [33918386], [12237818], [36768360], [-77890852], [-46101456], [-33494378], [1039552], [103147198], [-23242228], [52205690], [-41603498], [57855392], [32737902], [127681696], [24033204], [41038262], [-66405794], [-18866300], [31570080], [-109591968], [-53524386], [-85364044], [47453392], [113262774], [-156134564], [-93131184], [24499286], [57510330], [-96580656], [-73339240], [-69400936], [92737876], [84865288], [-145279554], [132648262], [128400480], [-92784072], [-127032548], [146864852], [132091572], [-67932338], [37756062], [-40023036], [196994894], [154088842], [-162609952], [131052038], [98051484], [-23980414], [40715398], [26013256], [-162126536], [-75132454], [176627162], [-17924562], [-28837732], [-148547934], [-133832410], [25716820], [-7891374], [114087450], [-22895500], [-90431682], [-222018400], [182801558], [-42767808], [-84288136], [198492654], [-83008550], [-217270932], [-52783776], [-66410880], [-236122056], [-128045294], [-42533292], [-27074980], [135132402], [-192306742], [-230167280], [282833574], [-91412862], [-59360988], [251668974], [62216948], [-33991906], [155815866], [131896480], [-109589382], [47141364], [205909932], [-187006042], [12031962], [-3092264], [253976016], [-211281916], [190784882], [44634736], [-70930382], [-108678076], [-82493194], [-227552784], [-56241182], [-103630970], [32875844], [-103806212], [242552878], [-108000396], [335331796], [-304252778], [-43674810], [-241815068], [-132007766], [-31452150], [-6771536], [73006726], [-56531306], [224240012], [2065004], [-318130612], [66118754], [-370354018], [-179442262], [-55989336], [-46086452], [139284916], [-147133180], [294752478], [145289318], [278823742], [164781560], [689126], [484469742], [480197610], [67836552], [383795754], [-230656512], [-300435206], [-419969188], [-424170144], [-279176958], [65924980], [-462092518], [-391158558], [508551552], [169949446], [34053334], [393706592], [497269788], [108647918], [106330148], [568567030], [-431177378], [-343841732], [391751480], [-298798612], [309441818], [-92583246], [319180176], [-428759702], [-573131800], [-434354308], [-138142356], [405118138], [-483708784], [102647950], [-593221308], [249971780], [139276430], [-653353010], [-510992642], [411996426], [417432454], [-193192232], [-377088978], [-232472164], [112487144], [516593456], [205995098], [-144870476], [-172191752], [-201230686], [-17301798], [341025752], [284797780], [-369625484], [-17071638], [-354891904], [183729574], [471660612], [-532441102], [432447466], [-499059522], [-397394552], [754519070], [-224748006], [-521982584], [667513726], [-343276674], [185899900], [184410028], [677466206], [-582153502], [-211841632], [-628253004], [262691114], [-588606542], [306546212], [687799616], [679532478], [691744064], [310856906], [457710754], [189747692], [193037196], [-374324090], [-64538038], [-323595080], [323885598], [746048388], [74033488]]
